@@ -1,8 +1,6 @@
 #' Data model class
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' The `dm` class holds a list of tables and their relationships.
 #' It is inspired by [datamodelr](https://github.com/bergant/datamodelr),
 #' and extends the idea by offering operations to access the data in the tables.
@@ -10,11 +8,10 @@
 #' `dm()` creates a `dm` object from [tbl] objects
 #' (tibbles or lazy data objects).
 #'
-#' @param ... Tables to add to the `dm` object.
-#'   If no names are provided, the tables
-#'   are auto-named.
-#' @param .name_repair Options for name repair.
-#'   Forwarded as `repair` to [vctrs::vec_as_names()].
+#' @param ... Tables or existing `dm` objects to add to the `dm` object.
+#'   Unnamed tables are auto-named, `dm` objects must not be named.
+#' @param .name_repair,.quiet Options for name repair.
+#'   Forwarded as `repair` and `quiet` to [vctrs::vec_as_names()].
 #'
 #' @return For `dm()`, `new_dm()`, `as_dm()`: A `dm` object.
 #'
@@ -22,10 +19,11 @@
 #'
 #' - [dm_from_con()] for connecting to all tables in a database
 #'   and importing the primary and foreign keys
+#' - [dm_get_tables()] for returning a list of tables
 #' - [dm_add_pk()] and [dm_add_fk()] for adding primary and foreign keys
 #' - [copy_dm_to()] for DB interaction
 #' - [dm_draw()] for visualization
-#' - [dm_join_to_tbl()] for flattening
+#' - [dm_flatten_to_tbl()] for flattening
 #' - [dm_filter()] for filtering
 #' - [dm_select_tbl()] for creating a `dm` with only a subset of the tables
 #' - [dm_nycflights13()]  for creating an example `dm` object
@@ -51,27 +49,48 @@
 #' dm_nycflights13()[["airports"]]
 #'
 #' dm_nycflights13() %>% names()
-#'
-#' dm_nycflights13() %>% dm_get_tables()
-dm <- function(..., .name_repair = c("check_unique", "unique", "universal", "minimal")) {
+dm <- function(...,
+               .name_repair = c("check_unique", "unique", "universal", "minimal"),
+               .quiet = FALSE) {
   quos <- enquos(...)
+  names <- names2(quos)
 
-  tbls <- map(quos, eval_tidy)
+  dots <- map(quos, eval_tidy)
 
-  if (has_length(quos)) {
-    src_index <- c(which(names(quos) == "src"), 1)[[1]]
-    if (is.src(tbls[[src_index]])) {
-      deprecate_soft("0.0.4.9001", "dm::dm(src = )")
-      return(exec(dm_from_con, con_from_src_or_con(tbls[[src_index]]), !!!tbls[-src_index]))
+  is_dm <- map_lgl(dots, is_dm)
+
+  for (i in which(is_dm)) {
+    if (names[[i]] != "") {
+      abort(c(
+        "All dm objects passed to `dm()` must be unnamed.",
+        i = paste0("Argument ", i, " has name ", tick(names[[i]]), ".")
+      ))
+    }
+
+    if (is_zoomed(dots[[i]])) {
+      abort(c(
+        "All dm objects passed to `dm()` must be unzoomed.",
+        i = paste0("Argument ", i, " is a zoomed dm.")
+      ))
     }
   }
 
-  names(tbls) <- vec_as_names(names(quos_auto_name(quos)), repair = .name_repair)
-  dm <- new_dm(tbls)
+  # FIXME: check not zoomed, prettier
+  stopifnot(names2(quos)[is_dm] == "")
+
+  dm_tbl <- dm_impl(dots[!is_dm], names(quos_auto_name(quos[!is_dm])))
+  def <- dm_bind_impl(c(dots[is_dm], list(dm_tbl)), .name_repair, .quiet, repair_arg = "")
+
+  dm <- new_dm3(def)
   dm_validate(dm)
   dm
 }
 
+dm_impl <- function(tbls, names) {
+  names(tbls) <- names
+  new_dm(tbls)
+}
+
 #' A low-level constructor
 #'
 #' @description
@@ -86,16 +105,47 @@ dm <- function(..., .name_repair = c("check_unique", "unique", "universal", "min
 #' @param tables A named list of the tables (tibble-objects, not names),
 #'   to be included in the `dm` object.
 #'
+#' @examples
+#' library(dm)
+#' library(nycflights13)
+#'
+#' # using `data.frame` objects
+#' new_dm(tibble::lst(weather, airports))
+#'
+#' # using `dm_keyed_tbl` objects
+#' dm <- dm_nycflights13()
+#' y1 <- dm$planes %>%
+#'   mutate() %>%
+#'   select(everything())
+#' y2 <- dm$flights %>%
+#'   left_join(dm$airlines, by = "carrier")
+#'
+#' new_dm(list("tbl1" = y1, "tbl2" = y2))
+#'
 #' @rdname dm
 #' @export
 new_dm <- function(tables = list()) {
-  new_dm2(tables)
+  def <- new_keyed_dm_def(tables)
+  new_dm3(def)
 }
 
-new_dm2 <- function(tables = list(),
-                    pks_df = tibble(table = character(), pks = list()),
-                    fks_df = tibble(table = character(), fks = list()),
-                    validate = TRUE) {
+new_keyed_dm_def <- function(tables = list()) {
+  is_keyed <- map_lgl(unname(tables), is_dm_keyed_tbl)
+  stopifnot(!anyDuplicated(names(tables)[is_keyed]))
+
+  # data should be saved as a tibble
+  unclassed_tables <- map(tables, unclass_keyed_tbl)
+
+  pks_df <- pks_df_from_keys_info(tables[is_keyed])
+  fks_df <- fks_df_from_keys_info(tables[is_keyed])
+
+  new_dm_def(unclassed_tables, pks_df, fks_df)
+}
+
+
+new_dm_def <- function(tables = list(),
+                       pks_df = tibble(table = character(), pks = list()),
+                       fks_df = tibble(table = character(), fks = list())) {
   # Legacy
   data <- unname(tables)
   table <- names2(tables)
@@ -120,17 +170,27 @@ new_dm2 <- function(tables = list(),
     mutate(fks = as_list_of(map(fks, `%||%`, new_fk()), .ptype = new_fk())) %>%
     mutate(filters = list_of(new_filter())) %>%
     left_join(zoom, by = "table") %>%
-    left_join(col_tracker_zoom, by = "table")
+    left_join(col_tracker_zoom, by = "table") %>%
+    mutate(uuid = vec_new_uuid_along(table))
 
-  new_dm3(def, validate = validate)
+  def
 }
 
 new_dm3 <- function(def, zoomed = FALSE, validate = TRUE) {
+  if (is.null(def[["uuid"]])) {
+    def$uuid <- vec_new_uuid_along(def$table)
+  } else {
+    missing <- which(is.na(def$uuid))
+    if (length(missing) > 0) {
+      def$uuid[missing] <- vec_new_uuid_along(missing)
+    }
+  }
+
   class <- c(
-    if (zoomed) "zoomed_dm",
+    if (zoomed) "dm_zoomed",
     "dm"
   )
-  out <- structure(list(def = def), class = class, version = 2L)
+  out <- structure(list(def = def), class = class, version = 3L)
 
   # Enable for strict tests (search for INSTRUMENT in .github/workflows):
   # if (validate) { dm_validate(out) } # INSTRUMENT: validate
@@ -143,9 +203,10 @@ dm_get_def <- function(x, quiet = FALSE) {
   # Most callers already call it, but not all
   check_dm(x)
 
-  if (!identical(attr(x, "version"), 2L)) {
+  if (!identical(attr(x, "version"), 3L)) {
     x <- dm_upgrade(x, quiet)
   }
+
   unclass(x)$def
 }
 
@@ -154,8 +215,18 @@ new_pk <- function(column = list()) {
   tibble(column = column)
 }
 
-new_fk <- function(ref_column = list(), table = character(), column = list(), on_delete = character()) {
-  stopifnot(is.list(column), is.list(ref_column), length(table) == length(column), length(table) == length(ref_column), length(on_delete) %in% c(1L, length(table)))
+new_fk <- function(ref_column = list(),
+                   table = character(),
+                   column = list(),
+                   on_delete = character()) {
+  stopifnot(
+    is.list(column),
+    is.list(ref_column),
+    length(table) == length(column),
+    length(table) == length(ref_column),
+    length(on_delete) %in% c(1L, length(table))
+  )
+
   tibble(ref_column, table, column, on_delete)
 }
 
@@ -176,28 +247,6 @@ new_col_tracker_zoom <- function() {
   tibble(table = character(), col_tracker_zoom = list())
 }
 
-#' Get tables
-#'
-#' `dm_get_tables()` returns a named list of \pkg{dplyr} [tbl] objects
-#' of a `dm` object.
-#' Filtering expressions defined by [dm_filter()] are NOT evaluated at this stage.
-#' To get a filtered table, use [dm_apply_filters_to_tbl()], to apply filters to all tables use [dm_apply_filters()].
-#'
-#' @rdname dm
-#'
-#' @return For `dm_get_tables()`: A named list with the tables constituting the `dm`.
-#'
-#' @export
-dm_get_tables <- function(x) {
-  check_not_zoomed(x)
-  dm_get_tables_impl(x)
-}
-
-dm_get_tables_impl <- function(x, quiet = FALSE) {
-  def <- dm_get_def(x, quiet)
-  set_names(def$data, def$table)
-}
-
 unnest_pks <- function(def) {
   # Optimized
   pk_df <- tibble(
@@ -248,12 +297,12 @@ is_dm <- function(x) {
 #'
 #' @rdname dm
 #' @export
-as_dm <- function(x) {
+as_dm <- function(x, ...) {
   UseMethod("as_dm")
 }
 
 #' @export
-as_dm.default <- function(x) {
+as_dm.default <- function(x, ...) {
   if (!is.list(x) || is.object(x)) {
     abort(paste0("Can't coerce <", class(x)[[1]], "> to <dm>."))
   }
@@ -276,24 +325,24 @@ tbl_src <- function(x) {
 }
 
 #' @export
-as_dm.src <- function(x) {
+as_dm.src <- function(x, ...) {
   dm_from_con(con = con_from_src_or_con(x), table_names = NULL)
 }
 
 #' @export
-as_dm.DBIConnection <- function(x) {
+as_dm.DBIConnection <- function(x, ...) {
   dm_from_con(con = x, table_names = NULL)
 }
 
 #' @export
 print.dm <- function(x, ...) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   show_dm(x)
   invisible(x)
 }
 
 #' @export
-print.zoomed_dm <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
+print.dm_zoomed <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
   format(x, ..., n = NULL, width = NULL, n_extra = NULL)
 }
 
@@ -327,7 +376,7 @@ show_dm <- function(x) {
   cat_line("Primary keys: ", def_get_n_pks(def))
   cat_line("Foreign keys: ", def_get_n_fks(def))
 
-  filters <- dm_get_filters(x)
+  filters <- dm_get_filters_impl(x)
   if (nrow(filters) > 0) {
     cat_rule("Filters", col = "orange")
     walk2(filters$table, filters$filter, ~ cat_line(paste0(.x, ": ", as_label(.y))))
@@ -336,16 +385,16 @@ show_dm <- function(x) {
 
 #' @export
 format.dm <- function(x, ...) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   def <- dm_get_def(x)
   glue("dm: {def_get_n_tables(def)} tables, {def_get_n_columns(def)} columns, {def_get_n_pks(def)} primary keys, {def_get_n_fks(def)} foreign keys")
 }
 
 #' @export
-format.zoomed_dm <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
+format.dm_zoomed <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
   # so far only 1 table can be zoomed on
-  zoomed_df <- as_zoomed_df(x)
-  cat_line(format(zoomed_df, ..., n = n, width = width, n_extra = n_extra))
+  dm_zoomed_df <- as_dm_zoomed_df(x)
+  cat_line(format(dm_zoomed_df, ..., n = n, width = width, n_extra = n_extra))
   invisible(x)
 }
 
@@ -365,25 +414,25 @@ def_get_n_fks <- function(def) {
   sum(map_int(def$fks, vec_size))
 }
 
-as_zoomed_df <- function(x) {
+as_dm_zoomed_df <- function(x) {
   zoomed <- dm_get_zoom(x)
 
   # for tests
-  new_zoomed_df(
+  new_dm_zoomed_df(
     zoomed$zoom[[1]],
     name_df = zoomed$table
   )
 }
 
-new_zoomed_df <- function(x, ...) {
+new_dm_zoomed_df <- function(x, ...) {
   if (!is.data.frame(x)) {
-    return(structure(x, class = c("zoomed_df", class(x)), ...))
+    return(structure(x, class = c("dm_zoomed_df", class(x)), ...))
   }
   # need this in order to avoid star (from rownames, automatic from `structure(...)`)
   # in print method for local tibbles
   new_tibble(
     x,
-    class = c("zoomed_df", class(x), c("tbl_df", "tbl", "data.frame")),
+    class = c("dm_zoomed_df", class(x), c("tbl_df", "tbl", "data.frame")),
     nrow = nrow(x),
     ...
   )
@@ -392,7 +441,7 @@ new_zoomed_df <- function(x, ...) {
 # this is called from `tibble:::trunc_mat()`, which is called from `tibble::format.tbl()`
 # therefore, we need to have our own subclass but the main class needs to be `tbl`
 #' @export
-tbl_sum.zoomed_df <- function(x) {
+tbl_sum.dm_zoomed_df <- function(x) {
   c(
     structure(attr(x, "name_df"), names = "Zoomed table"),
     NextMethod()
@@ -400,19 +449,19 @@ tbl_sum.zoomed_df <- function(x) {
 }
 
 #' @export
-format.zoomed_df <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
+format.dm_zoomed_df <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
   NextMethod()
 }
 
 #' @export
 `$.dm` <- function(x, name) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   table <- dm_tbl_name(x, {{ name }})
   tbl_impl(x, table)
 }
 
 #' @export
-`$.zoomed_dm` <- function(x, name) {
+`$.dm_zoomed` <- function(x, name) {
   name <- ensym(name)
   eval_tidy(quo(`$`(tbl_zoomed(x), !!name)))
 }
@@ -423,14 +472,14 @@ format.zoomed_df <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
 }
 
 #' @export
-`[[.dm` <- function(x, id) {
-  # for both dm and zoomed_dm
+`[[.dm` <- function(x, id, ...) {
+  # for both dm and dm_zoomed
   if (is.numeric(id)) id <- src_tbls_impl(x)[id] else id <- as_string(id)
   tbl_impl(x, id, quiet = TRUE)
 }
 
 #' @export
-`[[.zoomed_dm` <- function(x, id) {
+`[[.dm_zoomed` <- function(x, id) {
   `[[`(tbl_zoomed(x, quiet = TRUE), id)
 }
 
@@ -447,8 +496,8 @@ format.zoomed_df <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
 }
 
 #' @export
-`[.zoomed_dm` <- function(x, id) {
-  # for both dm and zoomed_dm
+`[.dm_zoomed` <- function(x, id) {
+  # for both dm and dm_zoomed
   `[`(tbl_zoomed(x), id)
 }
 
@@ -460,12 +509,12 @@ format.zoomed_df <- function(x, ..., n = NULL, width = NULL, n_extra = NULL) {
 
 #' @export
 names.dm <- function(x) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   src_tbls_impl(x, quiet = TRUE)
 }
 
 #' @export
-names.zoomed_dm <- function(x) {
+names.dm_zoomed <- function(x) {
   names(tbl_zoomed(x, quiet = TRUE))
 }
 
@@ -477,12 +526,12 @@ names.zoomed_dm <- function(x) {
 
 #' @export
 length.dm <- function(x) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   length(src_tbls_impl(x, quiet = TRUE))
 }
 
 #' @export
-length.zoomed_dm <- function(x) {
+length.dm_zoomed <- function(x) {
   length(tbl_zoomed(x, quiet = TRUE))
 }
 
@@ -493,7 +542,7 @@ length.zoomed_dm <- function(x) {
 
 #' @export
 str.dm <- function(object, ...) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   object <-
     dm_get_def(object, quiet = TRUE) %>%
     select(table, pks, fks, filters)
@@ -501,7 +550,7 @@ str.dm <- function(object, ...) {
 }
 
 #' @export
-str.zoomed_dm <- function(object, ...) {
+str.dm_zoomed <- function(object, ...) {
   object <-
     dm_get_def(object, quiet = TRUE) %>%
     mutate(zoom = if_else(map_lgl(zoom, is_null), NA_character_, table)) %>%
@@ -509,14 +558,70 @@ str.zoomed_dm <- function(object, ...) {
   str(object)
 }
 
-tbl_impl <- function(dm, from, quiet = FALSE) {
-  out <- dm_get_tables_impl(dm, quiet)[[from]]
+keyed_tbl_impl <- function(dm, from) {
+  tbl_impl(dm, from, keyed = TRUE)
+}
 
-  if (is.null(out)) {
+tbl_impl <- function(dm, from, quiet = FALSE, keyed = FALSE) {
+  def <- dm_get_def(dm, quiet = quiet)
+  idx <- match(from, def$table)
+  if (is.na(idx)) {
     abort_table_not_in_dm(from, src_tbls_impl(dm))
   }
 
-  out
+  tbl_def_impl(def, idx, keyed)
+}
+
+tbl_def_impl <- function(def, idx, keyed) {
+  data <- def$data[[idx]]
+
+  if (!keyed) {
+    return(data)
+  }
+
+  uuid_lookup <- def[c("table", "uuid")]
+
+  pk_def <- def$pks[[idx]]
+  if (nrow(pk_def) > 0) {
+    pk <- pk_def$column[[1]]
+  } else {
+    pk <- NULL
+  }
+
+  fks_in_def <-
+    def$fks[[idx]] %>%
+    left_join(uuid_lookup, by = "table")
+
+  fks_in <- new_fks_in(
+    fks_in_def$uuid,
+    fks_in_def$column,
+    fks_in_def$ref_column
+  )
+
+  fks_out_def <-
+    map2_dfr(def$uuid, def$fks, ~ tibble(ref_uuid = .x, .y)) %>%
+    filter(table == !!def$table[[idx]]) %>%
+    select(ref_uuid, ref_column, column)
+
+  fks_out <- new_fks_out(
+    fks_out_def$column,
+    fks_out_def$ref_uuid,
+    fks_out_def$ref_column
+  )
+
+  new_keyed_tbl(
+    data,
+    pk = pk,
+    fks_in = fks_in,
+    fks_out = fks_out,
+    uuid = def$uuid[[idx]]
+  )
+}
+
+dm_get_keyed_tables_impl <- function(dm) {
+  def <- dm_get_def(dm)
+  tables <- map(seq_along(def$table), ~ tbl_def_impl(def, .x, keyed = TRUE))
+  set_names(tables, def$table)
 }
 
 src_tbls_impl <- function(dm, quiet = FALSE) {
@@ -526,8 +631,6 @@ src_tbls_impl <- function(dm, quiet = FALSE) {
 #' Materialize
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' `compute()` materializes all tables in a `dm` to new (temporary or permanent)
 #' tables on the database.
 #'
@@ -535,7 +638,7 @@ src_tbls_impl <- function(dm, quiet = FALSE) {
 #' Called on a `dm` object, these methods create a copy of all tables in the `dm`.
 #' Depending on the size of your data this may take a long time.
 #'
-#' @param x A `dm`.
+#' @inheritParams dm_get_tables
 #' @param ... Passed on to [compute()].
 #' @return A `dm` object of the same structure as the input.
 #' @name materialize
@@ -559,9 +662,9 @@ src_tbls_impl <- function(dm, quiet = FALSE) {
 #'   pull_tbl(districts) %>%
 #'   class()
 compute.dm <- function(x, ...) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   x %>%
-    dm_apply_filters() %>%
+    dm_apply_filters_impl() %>%
     dm_get_def() %>%
     mutate(data = map(data, compute, ...)) %>%
     new_dm3()
@@ -576,8 +679,8 @@ compute.dm <- function(x, ...) {
 #' @rdname materialize
 #' @export
 collect.dm <- function(x, ..., progress = NA) {
-  # for both dm and zoomed_dm
-  x <- dm_apply_filters(x)
+  # for both dm and dm_zoomed
+  x <- dm_apply_filters_impl(x)
 
   def <- dm_get_def(x)
 
@@ -587,7 +690,7 @@ collect.dm <- function(x, ..., progress = NA) {
 }
 
 #' @export
-collect.zoomed_dm <- function(x, ...) {
+collect.dm_zoomed <- function(x, ...) {
   message("Detaching table from dm, use `collect(pull_tbl())` instead to silence this message.")
 
   collect(pull_tbl(x))
@@ -596,13 +699,13 @@ collect.zoomed_dm <- function(x, ...) {
 
 # FIXME: what about 'dim.dm()'?
 #' @export
-dim.zoomed_dm <- function(x) {
+dim.dm_zoomed <- function(x) {
   # dm method provided by base
   dim(tbl_zoomed(x, quiet = TRUE))
 }
 
 #' @export
-dimnames.zoomed_dm <- function(x) {
+dimnames.dm_zoomed <- function(x) {
   # dm method provided by base
   dimnames(tbl_zoomed(x))
 }
@@ -613,7 +716,7 @@ tbl_vars.dm <- function(x) {
 }
 
 #' @export
-tbl_vars.zoomed_dm <- function(x) {
+tbl_vars.dm_zoomed <- function(x) {
   tbl_vars(tbl_zoomed(x))
 }
 
@@ -641,7 +744,8 @@ empty_dm <- function() {
       fks = list_of(new_fk()),
       filters = list_of(new_filter()),
       zoom = list(),
-      col_tracker_zoom = list()
+      col_tracker_zoom = list(),
+      uuid = character(),
     )
   )
 }
@@ -649,14 +753,13 @@ empty_dm <- function() {
 #' Retrieve a table
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' This generic has methods for both `dm` classes:
 #' 1. With `pull_tbl.dm()` you can chose which table of the `dm` you want to retrieve.
-#' 1. With `pull_tbl.zoomed_dm()` you will retrieve the zoomed table in the current state.
+#' 1. With `pull_tbl.dm_zoomed()` you will retrieve the zoomed table in the current state.
 #'
 #' @inheritParams dm_add_pk
-#' @param table One unquoted table name for `pull_tbl.dm()`, ignored for `pull_tbl.zoomed_dm()`.
+#' @param table One unquoted table name for `pull_tbl.dm()`, ignored for `pull_tbl.dm_zoomed()`.
+#' @inheritParams dm_get_tables
 #'
 #' @return The requested table
 #'
@@ -670,21 +773,25 @@ empty_dm <- function() {
 #'   dm_zoom_to(airports) %>%
 #'   pull_tbl()
 #' @export
-pull_tbl <- function(dm, table) {
+pull_tbl <- function(dm, table, ..., keyed = FALSE) {
   UseMethod("pull_tbl")
 }
 
 #' @export
-pull_tbl.dm <- function(dm, table) {
-  # for both dm and zoomed_dm
+pull_tbl.dm <- function(dm, table, ..., keyed = FALSE) {
+  # for both dm and dm_zoomed
   # FIXME: shall we issue a special error in case someone tries sth. like: `pull_tbl(dm_for_filter, c(t4, t3))`?
   table_name <- as_string(enexpr(table))
   if (table_name == "") abort_no_table_provided()
-  tbl_impl(dm, table_name)
+  tbl_impl(dm, table_name, keyed = keyed)
 }
 
 #' @export
-pull_tbl.zoomed_dm <- function(dm, table) {
+pull_tbl.dm_zoomed <- function(dm, table, ..., keyed = FALSE) {
+  if (isTRUE(keyed)) {
+    abort("`keyed = TRUE` not supported for zoomed dm objects.")
+  }
+
   table_name <- as_string(enexpr(table))
   zoomed <- dm_get_zoom(dm)
   if (table_name == "") {
@@ -702,25 +809,23 @@ pull_tbl.zoomed_dm <- function(dm, table) {
 
 #' @export
 as.list.dm <- function(x, ...) {
-  # for both dm and zoomed_dm
+  # for both dm and dm_zoomed
   dm_get_tables_impl(x)
 }
 
 #' @export
-as.list.zoomed_dm <- function(x, ...) {
+as.list.dm_zoomed <- function(x, ...) {
   as.list(tbl_zoomed(x))
 }
 
 #' Get a glimpse of your `dm` object
 #'
-#' @param x A `dm` object.
+#' @inheritParams dm_get_tables
 #' @param width Controls the maximum number of columns on a line used in
 #'   printing. If `NULL`, `getOption("width")` will be consulted.
 #' @param ... Passed to [pillar::glimpse()].
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' `glimpse()` provides an overview (dimensions, column data types, primary
 #' keys, etc.) of all tables included in the `dm` object. It will additionally
 #' print details about outgoing foreign keys for the child table.
@@ -739,11 +844,12 @@ as.list.zoomed_dm <- function(x, ...) {
 #' @export
 glimpse.dm <- function(x, width = NULL, ...) {
   glimpse_width <- width %||% getOption("width")
-
   table_names_list <- names(dm_get_tables_impl(x))
 
   print_glimpse_table_meta(x, glimpse_width)
-  print_rule_between_tables()
+  if (!is_empty(table_names_list)) {
+    print_rule_between_tables()
+  }
   walk(table_names_list, ~ print_glimpse_table(x, .x, glimpse_width, ...))
 
   invisible(x)
@@ -751,7 +857,7 @@ glimpse.dm <- function(x, width = NULL, ...) {
 
 #' @rdname glimpse.dm
 #' @export
-glimpse.zoomed_dm <- function(x, width = NULL, ...) {
+glimpse.dm_zoomed <- function(x, width = NULL, ...) {
   glimpse_width <- width %||% getOption("width")
 
   table_name <- dm_get_zoom(x)$table[[1]]
