@@ -1,8 +1,6 @@
 #' Add foreign keys
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' `dm_add_fk()` marks the specified `columns` as the foreign key of table `table` with
 #' respect to a key of table `ref_table`.
 #' Usually the referenced columns are a primary key in `ref_table`,
@@ -214,8 +212,6 @@ dm_get_fk2_impl <- function(dm, table_name, ref_table_name) {
 #' Get foreign key constraints
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' Get a summary of all foreign key relations in a [`dm`].
 #'
 #' @return A tibble with the following columns:
@@ -244,7 +240,7 @@ dm_get_all_fks <- function(dm, parent_table = NULL, ...) {
   dm_get_all_fks_impl(dm, parent_table)
 }
 
-dm_get_all_fks_impl <- function(dm, parent_table = NULL, ignore_on_delete = FALSE) {
+dm_get_all_fks_impl <- function(dm, parent_table = NULL, ignore_on_delete = FALSE, id = FALSE) {
   def <- dm_get_def(dm)
 
   sub_def <- def[c("table", "fks")]
@@ -259,14 +255,20 @@ dm_get_all_fks_impl <- function(dm, parent_table = NULL, ignore_on_delete = FALS
   names(flat) <- c("parent_table", "parent_key_cols", "child_table", "child_fk_cols", "on_delete")
   flat[[2]] <- new_keys(flat[[2]])
   flat[[4]] <- new_keys(flat[[4]])
-  flat[c(3:4, 1:2, if (!ignore_on_delete) 5L)]
+  out <- flat[c(3:4, 1:2, if (!ignore_on_delete) 5L)]
+  if (id) {
+    out <-
+      out %>%
+      group_by(child_table) %>%
+      mutate(id = paste0(child_table, "_", row_number())) %>%
+      ungroup()
+  }
+  out
 }
 
 #' Remove foreign keys
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
 #' `dm_rm_fk()` can remove either one reference between two tables, or multiple references at once (with a message).
 #' An error is thrown if no matching foreign key is found.
 #'
@@ -415,7 +417,7 @@ dm_rm_fk_impl <- function(dm, table_name, cols, ref_table_name, ref_cols) {
 
 #' Foreign key candidates
 #'
-#' @description `r lifecycle::badge("questioning")`
+#' @description `r lifecycle::badge("experimental")`
 #'
 #' Determine which columns would be good candidates to be used as foreign keys of a table,
 #' to reference the primary key column of another table of the [`dm`] object.
@@ -438,7 +440,7 @@ dm_rm_fk_impl <- function(dm, table_name, cols, ref_table_name, ref_cols) {
 #' - the error message triggered for unsuitable candidates that may include the types of mismatched columns
 #'
 #' @section Life cycle:
-#' These functions are marked "questioning" because we are not yet sure about
+#' These functions are marked "experimental" because we are not yet sure about
 #' the interface, in particular if we need both `dm_enum...()` and `enum...()`
 #' variants.
 #' Changing the interface later seems harmless because these functions are
@@ -484,20 +486,20 @@ dm_enum_fk_candidates <- function(dm, table, ref_table, ...) {
 #' @details `enum_fk_candidates()` works like `dm_enum_fk_candidates()` with the zoomed table as `table`.
 #'
 #' @rdname dm_enum_fk_candidates
-#' @param zoomed_dm A `dm` with a zoomed table.
+#' @param dm_zoomed A `dm` with a zoomed table.
 #' @export
-enum_fk_candidates <- function(zoomed_dm, ref_table, ...) {
+enum_fk_candidates <- function(dm_zoomed, ref_table, ...) {
   check_dots_empty()
-  check_zoomed(zoomed_dm)
-  check_no_filter(zoomed_dm)
+  check_zoomed(dm_zoomed)
+  check_no_filter(dm_zoomed)
 
-  table_name <- orig_name_zoomed(zoomed_dm)
-  ref_table_name <- dm_tbl_name(zoomed_dm, {{ ref_table }})
+  table_name <- orig_name_zoomed(dm_zoomed)
+  ref_table_name <- dm_tbl_name(dm_zoomed, {{ ref_table }})
 
-  ref_tbl_pk <- dm_get_pk_impl(zoomed_dm, ref_table_name)
+  ref_tbl_pk <- dm_get_pk_impl(dm_zoomed, ref_table_name)
 
-  ref_tbl <- dm_get_tables_impl(zoomed_dm)[[ref_table_name]]
-  enum_fk_candidates_impl(table_name, tbl_zoomed(zoomed_dm), ref_table_name, ref_tbl, ref_tbl_pk) %>%
+  ref_tbl <- dm_get_tables_impl(dm_zoomed)[[ref_table_name]]
+  enum_fk_candidates_impl(table_name, tbl_zoomed(dm_zoomed), ref_table_name, ref_tbl, ref_tbl_pk) %>%
     rename(columns = column) %>%
     mutate(columns = new_keys(columns))
 }
