@@ -95,7 +95,7 @@ dm_learn_from_db <- function(dest, dbname = NA, schema = NULL, name_format = "{t
     arrange(ordinal_position) %>%
     dm_update_zoomed() %>%
     dm_select_tbl(-table_constraints) %>%
-    dm_squash_to_tbl(key_column_usage) %>%
+    dm_flatten_to_tbl(key_column_usage, .recursive = TRUE) %>%
     select(constraint_catalog, constraint_schema, constraint_name, dm_name, column_name) %>%
     group_by(constraint_catalog, constraint_schema, constraint_name, dm_name) %>%
     summarize(pks = list(tibble(column = list(column_name)))) %>%
@@ -166,7 +166,8 @@ dm_learn_from_db <- function(dest, dbname = NA, schema = NULL, name_format = "{t
     summarize(fks = list(bind_rows(fks))) %>%
     ungroup()
 
-  new_dm2(tables, pks_df, fks_df)
+  def <- new_dm_def(tables, pks_df, fks_df)
+  new_dm3(def)
 }
 
 schema_if <- function(schema, table, con, dbname = NULL) {
