@@ -2,7 +2,7 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'left_join()'", {
   # for left join test the basic flattening also on all DBs
   expect_equivalent_tbl(
     expect_message_obj(dm_flatten_to_tbl(dm_for_flatten(), fact)),
-    result_from_flatten()
+    result_from_flatten_new()
   )
 
   # a one-table-dm
@@ -20,11 +20,11 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'left_join()'", {
   expect_equivalent_tbl(
     out,
     left_join(
-      fact_clean(),
-      dim_1_clean(),
+      fact_clean_new(),
+      dim_1_clean_new(),
       by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")
     ) %>%
-      left_join(dim_2_clean(), by = c("dim_2_key" = "dim_2_pk"))
+      left_join(dim_2_clean_new(), by = c("dim_2_key" = "dim_2_pk"))
   )
 
   # change order of parent tables
@@ -34,10 +34,10 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'left_join()'", {
   expect_equivalent_tbl(
     out,
     left_join(
-      fact_clean(), dim_2_clean(),
+      fact_clean_new(), dim_2_clean_new(),
       by = c("dim_2_key" = "dim_2_pk")
     ) %>%
-      left_join(dim_1_clean(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2"))
+      left_join(dim_1_clean_new(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2"))
   )
 
   # with grandparent table
@@ -63,9 +63,9 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'left_join()'", {
 test_that("`dm_flatten_to_tbl()` does the right things for 'inner_join()'", {
   out <- expect_message_obj(dm_flatten_to_tbl(
     dm_for_flatten(), fact,
-    join = inner_join
+    .join = inner_join
   ))
-  expect_equivalent_tbl(out, result_from_flatten())
+  expect_equivalent_tbl(out, result_from_flatten_new())
 })
 
 test_that("`dm_flatten_to_tbl()` does the right things for 'full_join()'", {
@@ -73,35 +73,35 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'full_join()'", {
   skip_if_src("maria")
   out <- expect_message_obj(dm_flatten_to_tbl(
     dm_for_flatten(), fact,
-    join = full_join
+    .join = full_join
   ))
   expect_equivalent_tbl(
     out,
-    fact_clean() %>%
-      full_join(dim_1_clean(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")) %>%
-      full_join(dim_2_clean(), by = c("dim_2_key" = "dim_2_pk")) %>%
-      full_join(dim_3_clean(), by = c("dim_3_key" = "dim_3_pk")) %>%
-      full_join(dim_4_clean(), by = c("dim_4_key" = "dim_4_pk"))
+    fact_clean_new() %>%
+      full_join(dim_1_clean_new(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")) %>%
+      full_join(dim_2_clean_new(), by = c("dim_2_key" = "dim_2_pk")) %>%
+      full_join(dim_3_clean_new(), by = c("dim_3_key" = "dim_3_pk")) %>%
+      full_join(dim_4_clean_new(), by = c("dim_4_key" = "dim_4_pk"))
   )
 })
 
 test_that("`dm_flatten_to_tbl()` does the right things for 'semi_join()'", {
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(dm_for_flatten(), fact, join = semi_join),
+    dm_flatten_to_tbl(dm_for_flatten(), fact, .join = semi_join),
     fact()
   )
 })
 
 test_that("`dm_flatten_to_tbl()` does the right things for 'anti_join()'", {
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(dm_for_flatten(), fact, join = anti_join),
+    dm_flatten_to_tbl(dm_for_flatten(), fact, .join = anti_join),
     fact() %>% filter(1 == 0)
   )
 })
 
 test_that("`dm_flatten_to_tbl()` does the right things for 'nest_join()'", {
   expect_dm_error(
-    dm_flatten_to_tbl(dm_for_flatten(), fact, join = nest_join),
+    dm_flatten_to_tbl(dm_for_flatten(), fact, .join = nest_join),
     class = "no_flatten_with_nest_join"
   )
 })
@@ -111,37 +111,50 @@ test_that("`dm_flatten_to_tbl()` does the right things for 'right_join()'", {
   skip_if_src("sqlite")
   expect_equivalent_tbl(
     expect_message_obj(expect_warning_obj(
-      dm_flatten_to_tbl(dm_for_flatten(), fact, join = right_join),
+      dm_flatten_to_tbl(dm_for_flatten(), fact, .join = right_join),
       "right_join"
     )),
-    fact_clean() %>%
-      right_join(dim_1_clean(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")) %>%
-      right_join(dim_2_clean(), by = c("dim_2_key" = "dim_2_pk")) %>%
-      right_join(dim_3_clean(), by = c("dim_3_key" = "dim_3_pk")) %>%
-      right_join(dim_4_clean(), by = c("dim_4_key" = "dim_4_pk"))
+    fact_clean_new() %>%
+      right_join(dim_1_clean_new(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2")) %>%
+      right_join(dim_2_clean_new(), by = c("dim_2_key" = "dim_2_pk")) %>%
+      right_join(dim_3_clean_new(), by = c("dim_3_key" = "dim_3_pk")) %>%
+      right_join(dim_4_clean_new(), by = c("dim_4_key" = "dim_4_pk"))
   )
 
   # change order of parent tables
   out <- expect_message_obj(dm_flatten_to_tbl(
     dm_for_flatten(), fact, dim_2, dim_1,
-    join = right_join
+    .join = right_join
   ))
   expect_equivalent_tbl(
     out,
     right_join(
-      fact_clean(),
-      dim_2_clean(),
+      fact_clean_new(),
+      dim_2_clean_new(),
       by = c("dim_2_key" = "dim_2_pk")
     ) %>%
-      right_join(dim_1_clean(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2"))
+      right_join(dim_1_clean_new(), by = c("dim_1_key_1" = "dim_1_pk_1", "dim_1_key_2" = "dim_1_pk_2"))
   )
 })
 
-test_that("`dm_squash_to_tbl()` does the right things", {
+test_that("`dm_squash_to_tbl()` is deprecated but still works", {
+  # with grandparent table
+  # left_join:
+  expect_deprecated(
+    expect_equivalent_tbl(
+      dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3),
+      tf_5() %>%
+        left_join(tf_4(), by = c("l" = "h")) %>%
+        left_join(tf_3(), by = c("j" = "f", "j1" = "f1"))
+    )
+  )
+})
+
+test_that("`dm_flatten_to_tbl(.recursive = TRUE)` does the right things", {
   # with grandparent table
   # left_join:
   expect_equivalent_tbl(
-    dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3),
+    dm_flatten_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, .recursive = TRUE),
     tf_5() %>%
       left_join(tf_4(), by = c("l" = "h")) %>%
       left_join(tf_3(), by = c("j" = "f", "j1" = "f1"))
@@ -150,26 +163,26 @@ test_that("`dm_squash_to_tbl()` does the right things", {
   # deeper hierarchy available and `auto_detect = TRUE`
   # for flatten: columns from tf_5 + tf_4 + tf_3 + tf_4_2 + tf_6 are combined in one table, 10 cols in total
   expect_identical(
-    ncol(dm_squash_to_tbl(dm_more_complex(), tf_5)),
+    ncol(dm_flatten_to_tbl(dm_more_complex(), tf_5, .recursive = TRUE)),
     12L
   )
 
 
   # semi_join:
   expect_dm_error(
-    dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, join = semi_join),
+    dm_flatten_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, .join = semi_join, .recursive = TRUE),
     class = "squash_limited"
   )
 
   # anti_join:
   expect_dm_error(
-    dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, join = anti_join),
+    dm_flatten_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, .join = anti_join, .recursive = TRUE),
     class = "squash_limited"
   )
 
   # fails when there is a cycle:
   expect_dm_error(
-    dm_squash_to_tbl(dm_for_filter_w_cycle(), tf_5),
+    dm_flatten_to_tbl(dm_for_filter_w_cycle(), tf_5, .recursive = TRUE),
     "no_cycles"
   )
 
@@ -178,7 +191,7 @@ test_that("`dm_squash_to_tbl()` does the right things", {
 
   # full_join:
   expect_equivalent_tbl(
-    dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, join = full_join),
+    dm_flatten_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, .join = full_join, .recursive = TRUE),
     tf_5() %>%
       full_join(tf_4(), by = c("l" = "h")) %>%
       full_join(tf_3(), by = c("j" = "f", "j1" = "f1"))
@@ -188,13 +201,13 @@ test_that("`dm_squash_to_tbl()` does the right things", {
 
   # right_join:
   expect_dm_error(
-    dm_squash_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, join = right_join),
+    dm_flatten_to_tbl(dm_more_complex(), tf_5, tf_4, tf_3, .join = right_join, .recursive = TRUE),
     class = "squash_limited"
   )
 })
 
 test_that("prepare_dm_for_flatten() works", {
-  # unfiltered with rename
+  # with rename
   out <- expect_message_obj(prepare_dm_for_flatten(
     dm_for_flatten(),
     c("fact", "dim_1", "dim_3"),
@@ -202,40 +215,10 @@ test_that("prepare_dm_for_flatten() works", {
   ))
   expect_equivalent_dm(
     out,
-    dm_select_tbl(dm_for_flatten(), fact, dim_1, dim_3) %>% dm_disambiguate_cols(quiet = TRUE)
+    dm_select_tbl(dm_for_flatten(), fact, dim_1, dim_3) %>% dm_disambiguate_cols(.quiet = TRUE)
   )
 
-  # filtered with rename
-  prep_dm <-
-    dm_for_flatten() %>%
-    dm_select_tbl(fact, dim_1, dim_3) %>%
-    #
-    dm_zoom_to(fact) %>%
-    filter(dim_1_key_1 > 7, dim_1_key_2 > !!LETTERS[7]) %>%
-    dm_update_zoomed() %>%
-    #
-    dm_zoom_to(dim_1) %>%
-    filter(dim_1_pk_1 > 7, dim_1_pk_2 > !!LETTERS[7]) %>%
-    dm_update_zoomed()
-
-  prep_dm_renamed <- dm_disambiguate_cols(prep_dm, quiet = TRUE)
-
-  out <- expect_message_obj(prepare_dm_for_flatten(
-    dm_filter(dm_for_flatten(), dim_1, dim_1_pk_1 > 7, dim_1_pk_2 > !!LETTERS[7]),
-    c("fact", "dim_1", "dim_3"),
-    gotta_rename = TRUE
-  ))
-  expect_equivalent_dm(out, prep_dm_renamed)
-
-  # filtered without rename
-  out <- prepare_dm_for_flatten(
-    dm_filter(dm_for_flatten(), dim_1, dim_1_pk_1 > 7, dim_1_pk_2 > !!LETTERS[7]),
-    c("fact", "dim_1", "dim_3"),
-    gotta_rename = FALSE
-  )
-  expect_equivalent_dm(out, prep_dm)
-
-  # unfiltered without rename
+  # without rename
   expect_equivalent_dm(
     prepare_dm_for_flatten(dm_for_flatten(), c("fact", "dim_1", "dim_3"), gotta_rename = FALSE),
     dm_select_tbl(dm_for_flatten(), fact, dim_1, dim_3)
@@ -308,18 +291,18 @@ test_that("tests with 'bad_dm' work", {
   skip_if_src("maria")
 
   # filtered `dm`
-  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1, a != 4)
+  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1 = (a != 4))
 
   expect_equivalent_tbl(
     dm_flatten_to_tbl(bad_filtered_dm, tbl_1),
-    dm_apply_filters(bad_filtered_dm) %>% dm_flatten_to_tbl(tbl_1)
+    bad_filtered_dm %>% dm_flatten_to_tbl(tbl_1)
   )
 
 
   # filtered `dm`
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(bad_filtered_dm, tbl_1, join = semi_join),
-    dm_apply_filters(bad_filtered_dm) %>% dm_flatten_to_tbl(tbl_1, join = semi_join)
+    dm_flatten_to_tbl(bad_filtered_dm, tbl_1, .join = semi_join),
+    bad_filtered_dm %>% dm_flatten_to_tbl(tbl_1, .join = semi_join)
   )
 
   skip_if_not_installed("nycflights13")
@@ -340,11 +323,11 @@ test_that("tests with 'bad_dm' work (2)", {
   # full & right join not available on SQLite and MariaDB
   skip_if_src("sqlite", "maria")
 
-  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1, a != 4)
+  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1 = (a != 4))
 
   # flatten bad_dm() (no referential integrity)
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_2, tbl_3, join = full_join),
+    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_2, tbl_3, .join = full_join),
     tbl_1() %>%
       full_join(tbl_2(), by = c("a" = "id", "x")) %>%
       full_join(tbl_3(), by = c("b" = "id"))
@@ -358,17 +341,11 @@ test_that("tests with 'bad_dm' work (3)", {
   # full & right join not available on SQLite
   skip_if_src("sqlite")
 
-  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1, a != 4)
-
-  # filtered `dm`
-  expect_dm_error(
-    dm_flatten_to_tbl(bad_filtered_dm, tbl_1, join = full_join),
-    class = "apply_filters_first_full_join"
-  )
+  bad_filtered_dm <- dm_filter(bad_dm(), tbl_1 = (a != 4))
 
   # flatten bad_dm() (no referential integrity)
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_2, tbl_3, join = right_join),
+    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_2, tbl_3, .join = right_join),
     tbl_1() %>%
       right_join(tbl_2(), by = c("a" = "id", "x")) %>%
       right_join(tbl_3(), by = c("b" = "id"))
@@ -376,15 +353,9 @@ test_that("tests with 'bad_dm' work (3)", {
 
   # flatten bad_dm() (no referential integrity); different order
   expect_equivalent_tbl(
-    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_3, tbl_2, join = right_join),
+    dm_flatten_to_tbl(bad_dm(), tbl_1, tbl_3, tbl_2, .join = right_join),
     tbl_1() %>%
       right_join(tbl_3(), by = c("b" = "id")) %>%
       right_join(tbl_2(), by = c("a" = "id", "x"))
   )
-
-  # filtered `dm`
-  expect_dm_error(
-    dm_flatten_to_tbl(bad_filtered_dm, tbl_1, join = right_join),
-    class = "apply_filters_first_right_join"
-  )
 })
