@@ -1,63 +1,90 @@
 # FIXME: #313: learn only from current source
 
-# produces a randomized schema name with a length of 4-10 characters
-# consisting of the symbols in `reservoir`
-random_schema <- function() {
-  reservoir <- c(letters, LETTERS, "'", "-", "_", as.character(0:9))
-  how_long <- sample(4:10, 1)
-  paste0(reservoir[sample(seq_len(length(reservoir)), how_long, replace = TRUE)], collapse = "")
-}
-
-schema_name <- random_schema()
-
 test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'public') and get_src_tbl_names() works?", {
-  skip_if_src_not(c("mssql", "postgres"))
-
-  # FIXME: COMPOUND: Need to fix implementation
-  skip_if_remote_src()
+  skip_if_schema_not_supported()
 
   # dm_learn_from_mssql() --------------------------------------------------
-  src_db <- my_test_src()
+  con_db <- my_test_con()
 
   # create an object on the MSSQL-DB that can be learned
   withr::defer(
     try(walk(
       remote_tbl_names,
-      ~ try(dbExecute(src_db$con, paste0("DROP TABLE ", .x)))
+      ~ try(dbExecute(con_db, paste0("DROP TABLE ", .x)))
     ))
   )
 
-  dm_for_filter_copied <- copy_dm_to(src_db, dm_for_filter(), temporary = FALSE, table_names = ~ DBI::SQL(unique_db_table_name(.x)))
+  dm_for_filter_copied <- copy_dm_to(con_db, dm_for_filter(), temporary = FALSE, table_names = ~ DBI::SQL(unique_db_table_name(.x)))
   order_of_deletion <- c("tf_2", "tf_1", "tf_5", "tf_6", "tf_4", "tf_3")
 
   remote_tbl_names <-
     map_chr(
       dm_get_tables(dm_for_filter_copied)[order_of_deletion],
       dbplyr::remote_name
-    )
+    ) %>%
+    SQL() %>%
+    DBI::dbUnquoteIdentifier(conn = con_db) %>%
+    map_chr(~ .x@name[["table"]])
+
+  remote_tbl_map <- set_names(remote_tbl_names, gsub("^(tf_.).*$", "\\1", remote_tbl_names))
 
   # test 'get_src_tbl_names()'
-  src_tbl_names <- sort(unname(gsub("^.*\\.", "", get_src_tbl_names(src_db))))
+  src_tbl_names <- sort(unname(gsub("^.*\\.", "", get_src_tbl_names(con_db))))
   expect_identical(
     # fail if there are other tables in the default schema
     src_tbl_names[grep("tf_._", src_tbl_names)],
-    sort(dbQuoteIdentifier(src_db$con, remote_tbl_names))
+    sort(dbQuoteIdentifier(con_db, remote_tbl_names))
   )
 
-  dm_db_learned_all <- expect_message(dm_from_src(src_db))
+  expect_snapshot({
+    dm_from_con(con_db)[integer()]
+  })
+})
 
-  # in case there happen to be other tables in schema "dbo" or "public"
+test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'public') and get_src_tbl_names() works?", {
+  skip_if_schema_not_supported()
+
+  # dm_learn_from_mssql() --------------------------------------------------
+  con_db <- my_test_con()
+
+  # create an object on the MSSQL-DB that can be learned
+  withr::defer(
+    try(walk(
+      remote_tbl_names,
+      ~ try(dbExecute(con_db, paste0("DROP TABLE ", .x)))
+    ))
+  )
+
+  dm_for_filter_copied <- copy_dm_to(con_db, dm_for_filter(), temporary = FALSE, table_names = ~ DBI::SQL(unique_db_table_name(.x)))
+  order_of_deletion <- c("tf_2", "tf_1", "tf_5", "tf_6", "tf_4", "tf_3")
+
+  remote_tbl_names <-
+    map_chr(
+      dm_get_tables(dm_for_filter_copied)[order_of_deletion],
+      dbplyr::remote_name
+    ) %>%
+    SQL() %>%
+    DBI::dbUnquoteIdentifier(conn = con_db) %>%
+    map_chr(~ .x@name[["table"]])
+
+  remote_tbl_map <- set_names(remote_tbl_names, gsub("^(tf_.).*$", "\\1", remote_tbl_names))
+
+  expect_silent(dm_db_learned_all <- dm_from_con(con_db, learn_keys = TRUE))
+
+  # Select and fix table names
   dm_db_learned <-
     dm_db_learned_all %>%
-    dm_select_tbl(!!!remote_tbl_names)
+    dm_select_tbl(!!!remote_tbl_map)
 
   expect_equivalent_dm(
     dm_db_learned,
-    dm_for_filter()[order_of_deletion]
+    dm_for_filter()[order_of_deletion],
+    # FIXME: Enable fetching of on_delete information
+    ignore_on_delete = TRUE
   )
 
   # learning without keys:
-  dm_db_learned_no_keys <- expect_silent(dm_from_src(src_db, learn_keys = FALSE))
+  expect_silent(dm_db_learned_no_keys <- dm_from_con(con_db, learn_keys = FALSE))
 
   # for learning from DB without learning the key relations
   dm_for_filter_no_keys <-
@@ -69,10 +96,10 @@ test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'publ
     ) %>%
     new_dm3()
 
-  # in case there happen to be other tables in schema "dbo" or "public"
+  # Select and fix table names
   dm_db_learned_no_keys <-
     dm_db_learned_no_keys %>%
-    dm_select_tbl(!!!remote_tbl_names)
+    dm_select_tbl(!!!remote_tbl_map)
 
   expect_equivalent_dm(
     dm_db_learned_no_keys,
@@ -82,26 +109,35 @@ test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'publ
 
 
 test_that("Learning from specific schema on MSSQL or Postgres works?", {
-  skip_if_src_not(c("mssql", "postgres"))
+  skip_if_schema_not_supported()
 
-  # FIXME: COMPOUND: Need to fix implementation
-  skip_if_remote_src()
+  # produces a randomized schema name with a length of 4-10 characters
+  # consisting of the symbols in `reservoir`
+  random_schema <- function() {
+    reservoir <- c(letters)
+    how_long <- 10
+    paste0(reservoir[sample(seq_len(length(reservoir)), how_long, replace = TRUE)], collapse = "")
+  }
 
-  src_db <- my_test_src()
-  con_db <- src_db$con
+  schema_name <- random_schema()
+
+  con_db <- my_test_con()
 
   schema_name_q <- DBI::dbQuoteIdentifier(con_db, schema_name)
 
   DBI::dbExecute(con_db, paste0("CREATE SCHEMA ", schema_name_q))
 
   dm_for_disambiguate_copied <- copy_dm_to(
-    src_db,
+    con_db,
     dm_for_disambiguate(),
     temporary = FALSE,
-    table_names = ~ DBI::SQL(paste0(schema_name_q, ".", .x))
+    schema = schema_name
   )
   order_of_deletion <- c("iris_3", "iris_2", "iris_1")
-  remote_tbl_names <- set_names(paste0(schema_name_q, ".\"", order_of_deletion, "\""), order_of_deletion)
+  remote_tbl_names <- set_names(
+    paste0(schema_name_q, ".\"", order_of_deletion, "\""),
+    order_of_deletion
+  )
 
   withr::defer({
     walk(
@@ -111,15 +147,19 @@ test_that("Learning from specific schema on MSSQL or Postgres works?", {
     try(dbExecute(con_db, paste0("DROP SCHEMA ", schema_name_q)))
   })
 
+  normalize_table_name <- function(x) {
+    tolower(gsub('["`]', "", x))
+  }
+
   # test 'get_src_tbl_names()'
   expect_identical(
-    sort(get_src_tbl_names(src_db, schema = schema_name)),
-    SQL(sort(remote_tbl_names))
+    sort(normalize_table_name(get_src_tbl_names(con_db, schema = schema_name))),
+    SQL(sort(normalize_table_name(remote_tbl_names)))
   )
 
   # learning with keys:
   dm_db_learned <-
-    dm_from_src(src_db, schema = schema_name, learn_keys = TRUE) %>%
+    dm_from_con(con_db, schema = schema_name, learn_keys = TRUE) %>%
     dm_select_tbl(!!!order_of_deletion)
 
   expect_equivalent_dm(
@@ -129,7 +169,7 @@ test_that("Learning from specific schema on MSSQL or Postgres works?", {
 
   # learning without keys:
   dm_db_learned_no_keys <-
-    expect_silent(dm_from_src(src_db, schema = schema_name, learn_keys = FALSE)) %>%
+    expect_silent(dm_from_con(con_db, schema = schema_name, learn_keys = FALSE)) %>%
     dm_select_tbl(!!!order_of_deletion)
 
   dm_for_disambiguate_no_keys <-
@@ -155,7 +195,7 @@ test_that("Learning from SQLite works (#288)?", {
 
   expect_equivalent_dm(
     src_sqlite() %>%
-      dm_from_src() %>%
+      dm_from_con() %>%
       dm_select_tbl(test) %>%
       collect(),
     dm(test = tibble(a = 1:3))
@@ -211,8 +251,7 @@ test_that("'schema_if()' works", {
 test_that("Learning from MSSQL (schema 'dbo') on other DB works?", {
   skip_if_src_not("mssql")
   # dm_learn_from_mssql() --------------------------------------------------
-  src_db <- my_test_src()
-  con_db <- src_db$con
+  con_db <- my_test_con()
 
   # delete database after test
   withr::defer({
@@ -241,7 +280,7 @@ test_that("Learning from MSSQL (schema 'dbo') on other DB works?", {
 
 
   # test 'get_src_tbl_names()'
-  src_tbl_names <- unname(get_src_tbl_names(src_db, dbname = "test_database_dm"))
+  src_tbl_names <- unname(get_src_tbl_names(con_db, dbname = "test_database_dm"))
   expect_identical(
     src_tbl_names,
     sort(DBI::SQL(paste0(
@@ -256,7 +295,7 @@ test_that("Learning from MSSQL (schema 'dbo') on other DB works?", {
     test_2 = tibble(c = c(1L, 1L, 1L, 5L, 4L), d = c(10L, 11L, 10L, 10L, 11L))
   )
 
-  expect_message(dm_db_learned <- dm_from_src(src_db, dbname = "test_database_dm"))
+  expect_message(dm_db_learned <- dm_from_con(con_db, dbname = "test_database_dm"))
   dm_learned <- dm_db_learned %>% collect()
   expect_equivalent_dm(
     dm_learned,
@@ -267,8 +306,8 @@ test_that("Learning from MSSQL (schema 'dbo') on other DB works?", {
 
   # learning without keys:
   dm_learned_no_keys <- expect_silent(
-    dm_from_src(
-      src_db,
+    dm_from_con(
+      con_db,
       dbname = "test_database_dm",
       learn_keys = FALSE
     ) %>%
@@ -285,8 +324,7 @@ test_that("Learning from MSSQL (schema 'dbo') on other DB works?", {
 test_that("Learning from a specific schema in another DB for MSSQL works?", {
   skip_if_src_not("mssql")
   # dm_learn_from_mssql() --------------------------------------------------
-  src_db <- my_test_src()
-  con_db <- src_db$con
+  con_db <- my_test_con()
 
   original_dbname <- attributes(con_db)$info$dbname
 
@@ -325,7 +363,7 @@ test_that("Learning from a specific schema in another DB for MSSQL works?", {
   })
 
   # test 'get_src_tbl_names()'
-  src_tbl_names <- unname(get_src_tbl_names(src_db, schema = "dm_test", dbname = "test_database_dm"))
+  src_tbl_names <- unname(get_src_tbl_names(con_db, schema = "dm_test", dbname = "test_database_dm"))
   expect_identical(
     src_tbl_names,
     sort(DBI::SQL(paste0(
@@ -340,7 +378,7 @@ test_that("Learning from a specific schema in another DB for MSSQL works?", {
     test_2 = tibble(c = c(1L, 1L, 1L, 5L, 4L), d = c(10L, 11L, 10L, 10L, 11L))
   )
 
-  expect_message(dm_db_learned <- dm_from_src(src_db, schema = "dm_test", dbname = "test_database_dm"))
+  expect_message(dm_db_learned <- dm_from_con(con_db, schema = "dm_test", dbname = "test_database_dm"))
   dm_learned <- dm_db_learned %>% collect()
   expect_equivalent_dm(
     dm_learned[c("test_1", "test_2")],
@@ -351,8 +389,8 @@ test_that("Learning from a specific schema in another DB for MSSQL works?", {
 
   # learning without keys:
   dm_learned_no_keys <- expect_silent(
-    dm_from_src(
-      src_db,
+    dm_from_con(
+      con_db,
       schema = "dm_test",
       dbname = "test_database_dm",
       learn_keys = FALSE
@@ -365,6 +403,84 @@ test_that("Learning from a specific schema in another DB for MSSQL works?", {
   )
 })
 
-# tests for compound keys -------------------------------------------------
+# Must be in the same file as the other learning tests
+# to avoid race conditions.
+test_that("dm_meta() contents", {
+  skip_if_schema_not_supported()
 
-# test is already done in test-dm-from-src.R
+  # produces a randomized schema name with a length of 4-10 characters
+  # consisting of the symbols in `reservoir`
+  random_schema <- function() {
+    reservoir <- c(letters)
+    how_long <- 10
+    paste0(reservoir[sample(seq_len(length(reservoir)), how_long, replace = TRUE)], collapse = "")
+  }
+
+  schema_name <- random_schema()
+
+  # dm_learn_from_mssql() --------------------------------------------------
+  con_db <- my_test_con()
+
+  schema_name_q <- DBI::dbQuoteIdentifier(con_db, schema_name)
+
+  DBI::dbExecute(con_db, paste0("CREATE SCHEMA ", schema_name_q))
+
+  # create an object on the MSSQL-DB that can be learned
+  withr::defer({
+    try(walk(
+      order_of_deletion,
+      ~ try(dbExecute(con_db, paste0("DROP TABLE ", schema_name_q, ".", .x, " CASCADE")))
+    ))
+    try(dbExecute(con_db, paste0("DROP SCHEMA ", schema_name_q)))
+  })
+
+  dm_for_filter_copied <- copy_dm_to(con_db, dm_for_filter(), temporary = FALSE, schema = schema_name)
+  order_of_deletion <- c("tf_2", "tf_1", "tf_5", "tf_6", "tf_4", "tf_3")
+
+  meta <- dm_meta(con_db, schema = schema_name)
+
+  constraints <- dm_examine_constraints(meta, progress = FALSE)
+  expect_true(all(constraints$is_key))
+
+  arrange_all_but_constraint_name <- function(.x) {
+    names <- setdiff(colnames(.x), "constraint_name")
+    .x %>%
+      arrange(!!!syms(names))
+  }
+
+  expect_snapshot({
+    meta %>%
+      dm_select_tbl(-schemata) %>%
+      dm_zoom_to(table_constraints) %>%
+      filter(constraint_type %in% c("PRIMARY KEY", "FOREIGN KEY")) %>%
+      dm_update_zoomed() %>%
+      dm_get_tables() %>%
+      map(select, -any_of("column_default"), -contains("catalog"), -contains("schema")) %>%
+      map(arrange_all_but_constraint_name) %>%
+      map(collect) %>%
+      map(~ if ("constraint_name" %in% colnames(.x)) {
+        .x %>% mutate(constraint_name = as.integer(forcats::fct_inorder(constraint_name)))
+      } else {
+        .x
+      }) %>%
+      map(arrange_all) %>%
+      jsonlite::toJSON(pretty = TRUE) %>%
+      gsub(schema_name, "schema_name", .) %>%
+      gsub('(_catalog": ")[^"]*(")', "\\1catalog\\2", .) %>%
+      writeLines()
+  })
+})
+
+test_that("dm_from_con() with mariaDB", {
+  skip_if_offline()
+  my_db <- RMariaDB::dbConnect(
+    RMariaDB::MariaDB(),
+    username = "guest",
+    password = "relational",
+    dbname = "Financial_ijs",
+    host = "relational.fit.cvut.cz"
+  )
+  expect_snapshot_output(my_dm <- dm_from_con(my_db))
+  expect_snapshot(dm::dm_get_all_fks(my_dm))
+  expect_snapshot(dm::dm_get_all_pks(my_dm))
+})
